# Copyright(c) 1999, Space Science and Engineering Center, UW-Madison
# Refer to "McIDAS Software Acquisition and Distribution Policies"
# in the file  mcidas/data/license.txt

# $Id: GraphicsColors.tcl,v 1.8 2004/04/19 14:55:12 tsmith Exp $

##########################################################################
# This module is used to turn graphics colors on or off and to configure #
# those colors as well.  Its a popup window     			 #
##########################################################################
#
#  Global Procedures
#	GraphicsColors -- main entry point -- pops up window to configure
#			  the colors.
#  Global Variables -- ColorFlags(Levels) ON/OFF(1/0) -- Show Graphics level
#		       ColorFlags(ShowName) ON/OFF(1/0) -- Show colors name
#			   --unsupported for now -- may not have a name
#
#------------------------------------------------------------------------#
# GraphicsColors { } -- routine to initialize all of the correct         #
#      graphics color settings						 #
#------------------------------------------------------------------------#
proc GraphicsColors { } \
  {
  global GraphicsColorsInitialized
  global ColorFlags
  global GraphicsLoopSet
  
  set GraphicsColorsInitialized 1
  set ColorFlags(Levels) 1
  set ColorFlags(LoopSet) 1
  set ColorFlags(ShowName) 1
  
  set GraphicsLoopSet $ColorFlags(LoopSet)
  
  if { [winfo exists .tlGraphicsColors] } { wm deiconify .tlGraphicsColors; focus .tlGraphicsColors; return }
  toplevel .tlGraphicsColors -relief raised -borderwidth 3
  wm title .tlGraphicsColors "Graphics Color Selection"

  frame .tlGraphicsColors.fmFrameSelect
    frame .tlGraphicsColors.fmFrameSelect.fmLeft
      LMCreateLoopButton .tlGraphicsColors.fmFrameSelect.fmLeft "Frame" YES
    # -- in proc gui_toolbar.gui a shadow is placed on LMActiveFrame
    # -- if it changes then the colors are updated
    pack .tlGraphicsColors.fmFrameSelect.fmLeft -side left -padx 5
    frame .tlGraphicsColors.fmFrameSelect.fmRight
      checkbutton .tlGraphicsColors.fmFrameSelect.fmRight.cbForLoop -text "Set For Loop" \
           -variable GraphicsLoopSet -onvalue 1 -offvalue 0
      pack .tlGraphicsColors.fmFrameSelect.fmRight.cbForLoop -side top -pady 5 	   
    pack .tlGraphicsColors.fmFrameSelect.fmRight -side right -padx 5
  pack .tlGraphicsColors.fmFrameSelect

  frame .tlGraphicsColors.fmColorButtons 
    GraphicsColorsUpdateFrame 
  pack .tlGraphicsColors.fmColorButtons -side top -padx 10 -pady 10
  
  frame .tlGraphicsColors.fmExit
    button .tlGraphicsColors.fmExit.btExit -text "Dismiss" -command { destroy .tlGraphicsColors }
    pack .tlGraphicsColors.fmExit.btExit -side top -pady 10
  pack .tlGraphicsColors.fmExit -side top
  
  bind .tlGraphicsColors.fmExit.btExit <Destroy> { unset GraphicsColorsInitialized }
  }
#------------------------------------------------------------------------#  
# GraphicsColorsUpdateFrame -- updates the color window buttons based on #
#                           -- the current frame number                  #
#------------------------------------------------------------------------#
proc GraphicsColorsUpdateFrame { } \
  {
  global GraphicsColorsInitialized
  global ColorFlags
  global LMActiveFrame
  global Var
  global ColorStatus
  
  if { ![info exists GraphicsColorsInitialized] } { return }
  set w .tlGraphicsColors.fmColorButtons
  #-- read the ColorFlags(Levels) option
    set showlevel $ColorFlags(Levels)
  
  # determine how many graphics levels that there are
    set GraphicsLevels [UCpeek [ucLocation number_graphics]]
  
  #--- create the color list
    set guout [open "|gu.k TAB " r]
    while {[gets $guout guline]>-1} \
      {
      set level [lindex $guline 0]
      if { $level>=0 && $level<=$GraphicsLevels } \
        {
        set red [string range $guline 36 38]
        set green [string range $guline 31 33]
        set blue [string range $guline 24 26]
        set Name [string range $guline 8 17]
	if { [string match "*disabled*" $guline] } \
	{
	    set ColorStatus($level) "OFF"
	} \
	else \
	{
	    set ColorStatus($level) "ON"
	}
	if { ![string length [string trim $Name]] } \
	  { set Name "USER" }
        set ColorList($level) [format "#%02x%02x%02x $Name" $red $green $blue]
        }
      }
    catch { close $guout }  
  if { ![winfo exists $w.fmLeft] }  \
    {
    # build the buttons and frame
    frame $w.fmLeft    
      if { $ColorFlags(ShowName) } { label $w.fmLeft.lbCaption -text "       On/Off" } \
      else { label $w.fmLeft.lbCaption -text "On/Off" }
      pack $w.fmLeft.lbCaption -side top
    pack $w.fmLeft -side left -pady 5 -padx 5
    if { $GraphicsLevels>8 } \
      { 
      frame $w.fmRight 
        if { $ColorFlags(ShowName) } { label $w.fmRight.lbCaption -text "       On/Off" } \
        else { label $w.fmRight.lbCaption -text "On/Off" }
        pack $w.fmRight.lbCaption -side top
      pack $w.fmRight -side right -pady 5 -padx 5
      }
    for { set i 1 } { $i <= $GraphicsLevels } { incr i } \
      {
      if { $i <= 8 || ($i <= 16 && $GraphicsLevels>16 ) } \
        {
        # -- build the frame to hold the three buttons
        frame $w.fmLeft.fm$i
        set t $w.fmLeft.fm$i
        }\
      else \
        {
        # -- build the frame to hold the three buttons
        frame $w.fmRight.fm$i
        set t $w.fmRight.fm$i
        }  
      #-- see if the button should be labeled or not
        if { $showlevel } { set tlabel $i } else { set tlabel "" } 
	label $t.lbColor -text $tlabel -width 2
	if { $ColorFlags(ShowName) } \
	  {
          button $t.btColor -text [lindex $ColorList($i) 1] -background [lindex $ColorList($i) 0] \
	                  -foreground [InverseColor [lindex $ColorList($i) 0]] -width 10 \
	                  -command "AlterColorLevel $i [lindex $ColorList($i) 0]" 
	  
	  } \
	else \
	  {  
          button $t.btColor -bitmap  @[bitmapLoc cursor_solid.xbm] -foreground [lindex $ColorList($i) 0] \
	                  -activeforeground [lindex $ColorList($i) 0] \
	                  -command "AlterColorLevel $i [lindex $ColorList($i) 0]" 
          }			  
        checkbutton $t.btVis -text "" -width 1 -command "GraphicsToggleMask $i " \
	           -variable Var($t) -onvalue 1 -offvalue 0 -highlightthickness 0

      #-- Set color checkbutton status
	set frame $LMActiveFrame
	if { [string match $ColorStatus($i) "OFF"] } \
	  {
	  set Var($t) 0
	  } \
	else \
	  {
	  set Var($t) 1
	  }

        button $t.btEG -text "Erase" -command "GraphicsRemoveLevel $i" 
        pack $t.lbColor -side left		       
        pack $t.btColor -side left -padx 5
        pack $t.btVis -side left
        pack $t.btEG -side left
        pack $t -side top -padx 4 
      }
    } \
  else \
    {  
    # -- Only need to reconfig the buttons
    for { set i 1 } { $i <= $GraphicsLevels } { incr i } \
      {
      if { $i <= 8 || ($i <= 16 && $GraphicsLevels>16 ) } \
        {
        set t $w.fmLeft.fm$i
        }\
      else \
        {
        set t $w.fmRight.fm$i
        }  
      #-- see if the button should be labeled or not
      if { $ColorFlags(ShowName) } \
	{
        $t.btColor config -text [lindex $ColorList($i) 1] -background [lindex $ColorList($i) 0] \
	                  -foreground [InverseColor [lindex $ColorList($i) 0]] \
	                  -command "AlterColorLevel $i [lindex $ColorList($i) 0]" 
        } \
      else \
	{ 
        $t.btColor config -foreground [lindex $ColorList($i) 0] \
	                  -activeforeground [lindex $ColorList($i) 0] \
	                  -command "AlterColorLevel $i [lindex $ColorList($i) 0]" 
        }	
      }
    }  
  }
  
#------------------------------------------------------------------------------#
# GraphicsToggleMask -- uses GU to set color mask to on or off		       #
#------------------------------------------------------------------------------#
proc GraphicsToggleMask { level } \
  {
  global GraphicsLoopSet
  global LMActiveLoop LMLoops
  global LMActiveFrame
  global ColorStatus
   
  #-- now send to McIDAS to alter it
  if { $LMActiveLoop>=0 && $GraphicsLoopSet } \
    {
    set doit ""
    foreach i $LMLoops($LMActiveLoop,O) { set doit "$doit [expr $i+1]" }
    } \
  else \
    {
    set doit $LMActiveFrame
    }
    
  set frame $LMActiveFrame
  if { $ColorStatus($level) == "OFF" } \
    {
    set new_status "ON"
    } \
  else \
    {
    set new_status "OFF"
    }
  
  foreach i $doit \
    { 
    #-- set to on or off?
    set frame [expr $i-1]
    set ColorStatus($level) $new_status
    RunCommand "GU $new_status $level $i $i" 
    }
  }
#------------------------------------------------------------------------------#
# GraphicsRemoveLevel -- erases this color level from the frame/loop           #
#------------------------------------------------------------------------------#
proc GraphicsRemoveLevel { level } \
  {
  global GraphicsLoopSet
  global LMActiveLoop LMLoops
  global LMActiveFrame
   
  #-- now send to McIDAS to alter it
  if { $LMActiveLoop>=0 && $GraphicsLoopSet } \
    {
    set doit ""
    foreach i $LMLoops($LMActiveLoop,O) { set doit "$doit [expr $i+1]" }
    } \
  else \
    {
    set doit $LMActiveFrame
    }
  
  foreach i $doit { RunCommand "EG $i $i LEV=$level" }
  }
  
#------------------------------------------------------------------------------#
# AlterColorLevel { LevelToAlter ColorToAlter } -- changes the color value in the graphics level
#		-- LevelToAlter -- graphics level to alter color in
#		-- ColorToAlter -- current color
#------------------------------------------------------------------------------#
proc AlterColorLevel { LevelToAlter ColorToAlter }\
  {
  global GraphicsLoopSet
  global LMActiveLoop LMLoops
  global LMActiveFrame
   
  #-- get the new color
  set NewColor [ ColorSelect $ColorToAlter ]
  #-- break into RGB values
  scan $NewColor  "#%02x%02x%02x" r g b  
  #-- now send to McIDAS to alter it
  if { $LMActiveLoop>=0 && $GraphicsLoopSet } \
    {
    set doit ""
    foreach i $LMLoops($LMActiveLoop,O) { set doit "$doit [expr $i+1]" }
    } \
  else \
    {
    set doit $LMActiveFrame
    }
  
  foreach i $doit { RunCommand "GU MAKE $LevelToAlter $b $g $r $i $i" }
  
  #--- Altering a color level impacts the map buttons so update them as well
    MapColorsUpdateFrame
    GraphicsColorsUpdateFrame 
  }
