# Copyright(c) 1998, Space Science and Engineering Center, UW-Madison
# Refer to "McIDAS Software Acquisition and Distribution Policies"
# in the file  mcidas/data/license.txt

# $Id: listimage.tcl,v 1.10 1998/10/13 20:26:41 dglo Exp $

# ListImage w
#
# Create a top-level window containing a listbox showing a bunch of
# enhancement table names.
#
# Arguments:
#
#    w		Name to use for new top-level window.
#

proc ListImage { fullname thevar {w .listimage}} {

#
#	Let user know this will take a while.
#
	showBusyStatus
#
#	Create a popup widget for listing enhancement table names.
#
	catch {destroy $w}
	toplevel $w
	wm title $w "Images"

	wm iconname $w "Images"
	wm minsize $w 1 1
	placepopup $w
	if {$thevar == "dest"} {
	   message $w.msg -font -Adobe-times-medium-r-normal--*-180* \
	    -aspect 900 \
	    -text "Review existing Images in Dataset"
	} else {
	   message $w.msg -font -Adobe-times-medium-r-normal--*-180* \
	    -aspect 900 \
	    -text "Double click to select Image."
	}
	frame $w.frame -borderwidth 10
	scrollbar $w.frame.scroll -relief sunken -command "$w.frame.list yview"
	listbox $w.frame.list -yscroll "$w.frame.scroll set" -relief sunken \
		-width 80 -height 10 -setgrid 1

	pack append $w.frame $w.frame.scroll {right filly} \
		$w.frame.list {left expand fill}
#
#	Get the table names and put them in the listbox.
#
	set dsall [format "%s.ALL" $fullname]

	set lalist [open "|imglist.k $dsall" r]
	gets $lalist junk
	gets $lalist lab1
	gets $lalist lab2
	label $w.label1 -width 80 -text $lab1 -anchor w
	gets $lalist junk
	label $w.label2 -width 80 -text $lab2 -anchor w
	while {[gets $lalist line] > -1} {

#
#		Don't put the Done message into the box.
#
		if {[string first Done $line] == -1} {
#
#			Don't put the linked message into the box.
#
			if { [string first linked $line] == -1} {
#
#				Extract the file name.  Insert in listbox.
#
				set filename [string range $line 0 80]
				$w.frame.list insert end $filename
			}
		}
	}
	catch {close $lalist}
#
#	Select a table by double clicking.
#
	if {$thevar != "dest"} {
	   bind $w.frame.list <Double-1> \
		"addpos2imagename $fullname $thevar"
	}
#
#	Dismiss button.
#
	button $w.ok -text Dismiss -command "destroy $w" -width 20

	pack  $w.msg -side top -fill both
	pack $w.label1 -side top
	pack $w.label2 -side top
	pack $w.frame -side top
	pack $w.ok -side bottom
#
#	Let user know we're done.
#
	showIdleStatus
}

proc addpos2imagename { fullname thevar} {
	set areal [selection get]
	set pos [string range $areal 0 5]
	set pos [string trim $pos]
	global $thevar
	set $thevar [format "%s.%s" $fullname $pos]
	destroy .listimage
}
