#!/bin/sh

# Copyright(c) 1998, Space Science and Engineering Center, UW-Madison
# Refer to "McIDAS Software Acquisition and Distribution Policies"
# in the file  mcidas/data/license.txt

# $Id: popuphelplite.tcl,v 1.13 2001/10/05 17:47:17 tsmith Exp $

# the next line restarts using mcwish \
exec mcwish "$0" "$@"

# This code is an amagamation of the Unidata "tooltips" code
# and the balloon help package by Victor Wagner

# set timer for popup_help box
#
proc show_popup_help {widget {item {}}} {
  global popup_help_option popup_help_text
  global popup_help_after_ID

  if ![info exists popup_help_text($widget)] {
    return
  }

  if [string length $item] {
    set index "$widget,$item"
  } else {
    set index $widget
  }

  set popup_help_after_ID \
      [after $popup_help_option(delay) \
	   "create_popup_help $widget $popup_help_text($index)"]
}

# make sure the popup_help window isn't going to popup
#
proc hide_popup_help {} {
  global popup_help_after_ID

  # if popup_help timer is active, cancel it
  #
  if [info exists popup_help_after_ID] {
    after cancel $popup_help_after_ID
    unset popup_help_after_ID
  }

  # if popup_help window exists, make sure it's hidden
  #
  if [winfo exists .popup_help] {
    wm withdraw .popup_help
  }
}

# build and display popup_help window for the specified widget
# 
proc create_popup_help { widget args } {
  global popup_help_option

  # if this widget doesn't even exist, don't bother continuing
  #
  if ![winfo exists $widget] {
    return
  }

  # get current position of mouse in widget
  #
  set mx [winfo pointerx $widget]
  set my [winfo pointery $widget]

  # make sure the mouse is in this widget
  #
  set has_mouse [winfo containing $mx $my]
  if {$has_mouse != $widget } {
    return
  }

  # if the popup_help window doesn't exist, create it
  #
  if ![winfo exists .popup_help] {
    toplevel .popup_help
    wm override .popup_help 1
    label .popup_help.text \
	-relief raised \
	-bd 2 \
	-bg $popup_help_option(background) \
	-fg $popup_help_option(foreground) \
	-text $args
    pack  .popup_help.text -ipadx 10

  # otherwise, just refresh the text in the popup_help window
  #
  } else {
    .popup_help.text configure -text $args
  }

  # make sure the widget is still there...
  #
  update
  if ![winfo exists $widget] {
    return
  }

  # Position window a little bit down and to right of widget.
  #
  set x [winfo rootx $widget]
  set y [expr [winfo rooty $widget] + [winfo height $widget]]
  set w [winfo width .popup_help.text]
  set W [expr [winfo rootx .] + [winfo width .]]
#ml  if { [expr $x + $w] > $W } {
#ml    set x [expr $W - $w]
#ml  }
  wm geometry .popup_help "+$x+$y"

  # Show window.
  #
  wm deiconify .popup_help
  raise .popup_help
}

# enable popup_help for this class or widget path
#
proc enable_popup_help {class {script {}}} {
  if [llength script] {
    set extra " \[$script\]"
  } else {
    set extra ""
  }
  bind $class <Enter> "+show_popup_help %W $extra"
  bind $class <Leave> "+hide_popup_help"
  bind $class <Button-1> "+hide_popup_help"
}

# initialize the popup_help subsystem
#
proc init_popup_help {} {
  global popup_help_option

  if ![info exists popup_help_option(delay)] {
    set popup_help_option(delay) 500            ;# half second delay
    set popup_help_option(background) "#ffffaa"  ;# yellow
    set popup_help_option(foreground) black
  }

  enable_popup_help Button
  enable_popup_help Menubutton
  enable_popup_help Menu "%W index active"
  enable_popup_help Scale
  enable_popup_help Checkbutton
}

# simple test for the popup_help subsystem
#
proc test_popup_help {} {
  global popup_help_text

  init_popup_help
  button .foo -text Foo -command {
    set str [.foo cget -text]
    switch $str {
      Foo {
	.foo configure -text Bar
	set popup_help_text(.foo) "BarFoo"
      }
      Bar {
	.foo configure -text Foo
	set popup_help_text(.foo) "FooBar"
      }
    }
  }
  pack .foo -side top -expand 1 -fill x
  set popup_help_text(.foo) "FooBar"

  button .quit -text Quit -command {destroy .}
  pack .quit -side bottom -expand 1 -fill x
  set popup_help_text(.quit) "Quit test"
}

# test_popup_help

package provide popup_help 1.0
